/*  BeroList Mailing List - Configuration file                    */
/*  (c) 1996 by Bernhard Rosenkraenzer, root@startrek.in-trier.de */

/* VERSION will be used in error messages */
#define VERSION "BeroList 2.0.0"

/* Comment out if you do not want BeroList to create log files */
/* #define LOG */

/* Location of logfiles - must be writable by bin */
#define LOGDIR "/usr/inet/lists/logs"

/* Directory of list members, configs, etc. - must be read-writable by bin */
#define LISTDIR "/usr/inet/lists/lists"

/* SMTP server used to send messages   */
/* localhost should work in most cases */
#define SMTP_SERVER "localhost"

/* Hostname for mailing list server
   If HOST is not defined, BeroList will try to determine it itself
   This is _NOT_ recommended. (see README) */
/* #define HOST "startrek.in-trier.de" */

/* Maximum number of members for a list */
#define MAX_MEMBERS 1000

/* Uncomment the following lines if you do not want sender and list addresses
   in your message */
/* #define NO_SENDER_ADDRESS */
/* #define NO_LIST_ADDRESS   */   

/* Path to sendmail - required if USE_SENDMAIL is defined. */
/* #define SENDMAIL "/usr/bin/sendmail"  */

/* If USE_SENDMAIL is defined, BeroList will use sendmail to send out
   messages (the 1.x way).                                            */
/* #define USE_SENDMAIL                                               */

/* Path to sendmail - required if USE_SENDMAIL is defined. */
/* #define SENDMAIL "/usr/bin/sendmail"  */

/* define TRUSTED if 'bin' may run sendmail -f - RECOMMENDED
   Not necessary if USE_SENDMAIL is undefined. */
/* #define TRUSTED */

/* That's it. Go on and run make.
   If you're using a 386, use make 386
                     486           486
                     Pentium       586 */
